"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ToIntegerOrInfinity = ToIntegerOrInfinity;
exports.LengthOfArrayLike = LengthOfArrayLike;
exports.SpeciesConstructor = SpeciesConstructor;
exports.defaultCompareFunction = defaultCompareFunction;

var _isObject = _interopRequireDefault(require("lodash/isObject.js"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @param {unknown} target
 * @returns {number}
 */
function ToIntegerOrInfinity(target) {
  const number = Number(target);

  if (Number.isNaN(number) || number === 0) {
    return 0;
  }

  if (number === Infinity) {
    return Infinity;
  }

  if (number === -Infinity) {
    return -Infinity;
  }

  return Math.trunc(number);
}
/**
 * @param {unknown} target
 * @returns {number}
 */


function ToLength(target) {
  const length = ToIntegerOrInfinity(target);

  if (length < 0) {
    return 0;
  }

  return length < Number.MAX_SAFE_INTEGER ? length : Number.MAX_SAFE_INTEGER;
}
/**
 * @param {object} arrayLike
 * @returns {number}
 */


function LengthOfArrayLike(arrayLike) {
  if (!(0, _isObject.default)(arrayLike)) {
    throw TypeError("this is not a object");
  }

  return ToLength(arrayLike.length);
}
/**
 * @param {object} target
 * @param {Function} defaultConstructor
 * @returns {Function}
 */


function SpeciesConstructor(target, defaultConstructor) {
  if (!(0, _isObject.default)(target)) {
    throw TypeError("this is not a object");
  }

  const constructor = target.constructor;

  if (constructor === undefined) {
    return defaultConstructor;
  }

  if (!(0, _isObject.default)(constructor)) {
    throw TypeError("constructor is not a object");
  }

  const species = constructor[Symbol.species];

  if (species == null) {
    return defaultConstructor;
  }

  return species;
}
/**
 * @param {number} x
 * @param {number} y
 * @returns {-1 | 0 | 1}
 */


function defaultCompareFunction(x, y) {
  const [isNaN_x, isNaN_y] = [Number.isNaN(x), Number.isNaN(y)];

  if (isNaN_x && isNaN_y) {
    return 0;
  }

  if (isNaN_x) {
    return 1;
  }

  if (isNaN_y) {
    return -1;
  }

  if (x < y) {
    return -1;
  }

  if (x > y) {
    return 1;
  }

  if (x === 0 && y === 0) {
    const [isPlusZero_x, isPlusZero_y] = [Object.is(x, 0), Object.is(y, 0)];

    if (!isPlusZero_x && isPlusZero_y) {
      return -1;
    }

    if (isPlusZero_x && !isPlusZero_y) {
      return 1;
    }
  }

  return 0;
}